@@ -276,11 +276,6 @@ decode_sequence (GstVaapiDecoderVC1 * decoder, GstVC1BDU * rbdu,
 
   priv->has_entrypoint = FALSE;
 
-  if (adv_hdr->interlace != 0) {
-    GST_ERROR ("interlaced sequence unsupported");
-    return GST_VAAPI_DECODER_STATUS_ERROR_UNSUPPORTED_PROFILE;
-  }
-
   /* Reset POC */
   if (priv->last_non_b_picture) {
     if (priv->last_non_b_picture->poc == priv->next_poc)
@@ -962,6 +957,13 @@ decode_frame (GstVaapiDecoderVC1 * decoder, GstVC1BDU * rbdu, GstVC1BDU * ebdu)
     return get_status (result);
   }
 
+  /* @FIXME: intel-driver cannot handle interlaced frames */
+  if (priv->profile == GST_VAAPI_PROFILE_VC1_ADVANCED
+      && frame_hdr->pic.advanced.fcm != GST_VC1_FRAME_PROGRESSIVE) {
+    GST_ERROR ("interlaced video not supported");
+    return GST_VAAPI_DECODER_STATUS_ERROR_UNSUPPORTED_PROFILE;
+  }
+
   switch (frame_hdr->ptype) {
     case GST_VC1_PICTURE_TYPE_I:
       picture->type = GST_VAAPI_PICTURE_TYPE_I;
@@ -1303,6 +1305,10 @@ gst_vaapi_decoder_vc1_parse (GstVaapiDecoder * base_decoder,
     case GST_VC1_SLICE:
       flags |= GST_VAAPI_DECODER_UNIT_FLAG_SLICE;
       break;
+    case GST_VC1_FIELD:
+      /* @FIXME: intel-driver cannot handle interlaced frames */
+      GST_ERROR ("interlaced video not supported");
+      return GST_VAAPI_DECODER_STATUS_ERROR_UNSUPPORTED_PROFILE;
   }
   GST_VAAPI_DECODER_UNIT_FLAG_SET (unit, flags);
   return GST_VAAPI_DECODER_STATUS_SUCCESS;
@@ -1398,7 +1404,8 @@ gst_vaapi_decoder_vc1_flush (GstVaapiDecoder * base_decoder)
   GstVaapiDecoderVC1 *const decoder = GST_VAAPI_DECODER_VC1_CAST (base_decoder);
   GstVaapiDecoderVC1Private *const priv = &decoder->priv;
 
-  gst_vaapi_dpb_flush (priv->dpb);
+  if (priv->is_opened)
+    gst_vaapi_dpb_flush (priv->dpb);
   return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
