@@ -131,8 +131,14 @@ typedef enum {
  *   H.265 main 10 profile [A.3.3]
  * @GST_VAAPI_PROFILE_H265_MAIN_STILL_PICTURE:
  *   H.265 main still picture profile [A.3.4]
- * @GST_VAAPI_PROFILE_VP9:
- *   VP9 prfile 0
+ * @GST_VAAPI_PROFILE_VP9_0:
+ *   VP9 prfile 0, bitdepth=8, 420
+ * @GST_VAAPI_PROFILE_VP9_1:
+ *   VP9 prfile 1, bitdepth=8, 422/444/440/RGB
+ * @GST_VAAPI_PROFILE_VP9_2:
+ *   VP9 prfile 2, bitdepth=10/12, 420
+ * @GST_VAAPI_PROFILE_VP9_3:
+ *   VP9 prfile 3 bitdepth=10/12, 422/444/440/RGB
  *
  * The set of all profiles for #GstVaapiProfile.
  */
@@ -169,7 +175,10 @@ typedef enum {
     GST_VAAPI_PROFILE_H265_MAIN10           = GST_VAAPI_MAKE_PROFILE(H265,2),
     GST_VAAPI_PROFILE_H265_MAIN_STILL_PICTURE =
                                                GST_VAAPI_MAKE_PROFILE(H265,3),
-    GST_VAAPI_PROFILE_VP9                   = GST_VAAPI_MAKE_PROFILE(VP9,1),
+    GST_VAAPI_PROFILE_VP9_0                   = GST_VAAPI_MAKE_PROFILE(VP9,1),
+    GST_VAAPI_PROFILE_VP9_1                   = GST_VAAPI_MAKE_PROFILE(VP9,2),
+    GST_VAAPI_PROFILE_VP9_2                   = GST_VAAPI_MAKE_PROFILE(VP9,3),
+    GST_VAAPI_PROFILE_VP9_3                   = GST_VAAPI_MAKE_PROFILE(VP9,4),
 } GstVaapiProfile;
 
 /**
@@ -179,6 +188,8 @@ typedef enum {
  * @GST_VAAPI_ENTRYPOINT_MOCO: Motion Compensation
  * @GST_VAAPI_ENTRYPOINT_SLICE_ENCODE: Encode Slice
  * @GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE: Encode Picture
+ * @GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP: Encode Slice low power/
+ *                                        high performace varient
  *
  * The set of all entrypoints for #GstVaapiEntrypoint
  */
@@ -187,7 +198,8 @@ typedef enum {
     GST_VAAPI_ENTRYPOINT_IDCT,
     GST_VAAPI_ENTRYPOINT_MOCO,
     GST_VAAPI_ENTRYPOINT_SLICE_ENCODE,
-    GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE
+    GST_VAAPI_ENTRYPOINT_PICTURE_ENCODE,
+    GST_VAAPI_ENTRYPOINT_SLICE_ENCODE_LP
 } GstVaapiEntrypoint;
 
 const gchar *
