@@ -159,7 +159,10 @@ gst_vaapi_surface_create_full (GstVaapiSurface * surface,
   extbuf.pixel_format = va_format->fourcc;
   extbuf.width = GST_VIDEO_INFO_WIDTH (vip);
   extbuf.height = GST_VIDEO_INFO_HEIGHT (vip);
-  extbuf_needed = ! !(flags & GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE);
+  if (flags & GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE) {
+    extbuf.flags &= ~VA_SURFACE_EXTBUF_DESC_ENABLE_TILING;
+    extbuf_needed = TRUE;
+  }
 
   extbuf.num_planes = GST_VIDEO_INFO_N_PLANES (vip);
   if (flags & GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_STRIDES) {
@@ -1013,6 +1016,10 @@ gst_vaapi_surface_set_subpictures_from_composition (GstVaapiSurface * surface,
         (gint *) & sub_rect.x, (gint *) & sub_rect.y,
         &sub_rect.width, &sub_rect.height);
 
+    /* ensure that the overlay is not bigger than the surface */
+    sub_rect.y = MIN (sub_rect.y, surface->height);
+    sub_rect.width = MIN (sub_rect.width, surface->width);
+
     if (!gst_vaapi_surface_associate_subpicture (surface, subpicture,
             NULL, &sub_rect)) {
       GST_WARNING ("could not render overlay rectangle %p", rect);
