@@ -113,6 +113,8 @@ gst_vaapi_rotation_get_type (void)
         "Rotated by 180°, clockwise", "180"},
     {GST_VAAPI_ROTATION_270,
         "Rotated by 270°, clockwise", "270"},
+    {GST_VAAPI_ROTATION_AUTOMATIC,
+        "Rotated by image-orientating tag°", "Automatic"},
     {0, NULL, NULL},
   };
 
@@ -143,6 +145,8 @@ gst_vaapi_rate_control_get_type (void)
         "Variable bitrate", "vbr"},
     {GST_VAAPI_RATECONTROL_VBR_CONSTRAINED,
         "Variable bitrate - Constrained", "vbr_constrained"},
+    {GST_VAAPI_RATECONTROL_MB,
+        "Macroblock based rate control", "mb"},
     {0, NULL, NULL},
   };
 
