@@ -26,9 +26,9 @@
  * @short_description: A VA-API based video decoder with a
  * post-processor
  *
- * vaapidecodebin is similar #GstVaapiDecode, but it is composed by
- * the vaapidecode, a #GstQueue, and the #GstVaapiPostproc, if it is
- * available and functional in the setup.
+ * vaapidecodebin is similar vaapi{CODEC}dec, but it is composed by
+ * the unregistered vaapidecode, a #GstQueue, and the
+ * #GstVaapiPostproc, if it is available and functional in the setup.
  *
  * It offers the functionality of #GstVaapiDecode and the many options
  * of #GstVaapiPostproc.
@@ -73,13 +73,6 @@ enum
   PROP_LAST
 };
 
-enum
-{
-  HAS_VPP_UNKNOWN,
-  HAS_VPP_NO,
-  HAS_VPP_YES
-};
-
 static GParamSpec *properties[PROP_LAST];
 
 /* Default templates */
@@ -109,8 +102,10 @@ static const char gst_vaapi_decode_bin_sink_caps_str[] =
 static const char gst_vaapi_decode_bin_src_caps_str[] =
   GST_VAAPI_MAKE_SURFACE_CAPS ", "
   GST_CAPS_INTERLACED_FALSE "; "
+#if (USE_GLX || USE_EGL)
   GST_VAAPI_MAKE_GLTEXUPLOAD_CAPS ", "
   GST_CAPS_INTERLACED_FALSE "; "
+#endif
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
   GST_CAPS_INTERLACED_FALSE;
 /* *INDENT-ON* */
@@ -135,116 +130,13 @@ post_missing_element_message (GstVaapiDecodeBin * vaapidecbin,
 {
   GstMessage *msg;
 
-  GST_ERROR_OBJECT (vaapidecbin, "Failed to create %s element",
-      missing_factory);
-  msg =
-      gst_missing_element_message_new (GST_ELEMENT_CAST (vaapidecbin),
+  msg = gst_missing_element_message_new (GST_ELEMENT_CAST (vaapidecbin),
       missing_factory);
   gst_element_post_message (GST_ELEMENT_CAST (vaapidecbin), msg);
-}
 
-static gboolean
-activate_vpp (GstVaapiDecodeBin * vaapidecbin)
-{
-  GstPad *queue_srcpad, *srcpad, *vpp_sinkpad, *vpp_srcpad;
-  gboolean res;
-
-  if (vaapidecbin->postproc)
-    return TRUE;
-
-  if (vaapidecbin->has_vpp != HAS_VPP_YES || vaapidecbin->disable_vpp)
-    return TRUE;
-
-  GST_DEBUG_OBJECT (vaapidecbin, "Enabling VPP");
-
-  /* create the postproc */
-  vaapidecbin->postproc =
-      gst_element_factory_make ("vaapipostproc", "vaapipostproc");
-  if (!vaapidecbin->postproc)
-    goto error_element_missing;
-
-  g_object_set (G_OBJECT (vaapidecbin->postproc), "deinterlace-method",
-      vaapidecbin->deinterlace_method, NULL);
-
-  gst_bin_add (GST_BIN (vaapidecbin), vaapidecbin->postproc);
-
-  if (!gst_element_sync_state_with_parent (vaapidecbin->postproc))
-    goto error_sync_state;
-
-  srcpad = gst_element_get_static_pad (GST_ELEMENT_CAST (vaapidecbin), "src");
-  if (!gst_ghost_pad_set_target (GST_GHOST_PAD_CAST (srcpad), NULL))
-    goto error_link_pad;
-
-  queue_srcpad = gst_element_get_static_pad (vaapidecbin->queue, "src");
-  vpp_sinkpad = gst_element_get_static_pad (vaapidecbin->postproc, "sink");
-  res = (gst_pad_link (queue_srcpad, vpp_sinkpad) == GST_PAD_LINK_OK);
-  gst_object_unref (vpp_sinkpad);
-  gst_object_unref (queue_srcpad);
-  if (!res)
-    goto error_link_pad;
-
-  vpp_srcpad = gst_element_get_static_pad (vaapidecbin->postproc, "src");
-  res = gst_ghost_pad_set_target (GST_GHOST_PAD_CAST (srcpad), vpp_srcpad);
-  gst_object_unref (vpp_srcpad);
-  if (!res)
-    goto error_link_pad;
-
-  gst_object_unref (srcpad);
-
-  return TRUE;
-
-error_element_missing:
-  {
-    post_missing_element_message (vaapidecbin, "vaapipostproc");
-    return FALSE;
-  }
-error_sync_state:
-  {
-    GST_ERROR_OBJECT (vaapidecbin, "Failed to sync VPP state");
-    return FALSE;
-  }
-error_link_pad:
-  {
-    gst_object_unref (srcpad);
-    GST_ERROR_OBJECT (vaapidecbin, "Failed to link the child elements");
-    return FALSE;
-  }
-}
-
-static gboolean
-ensure_vpp (GstVaapiDecodeBin * vaapidecbin)
-{
-  GstVaapiDisplay *display;
-
-  if (vaapidecbin->has_vpp != HAS_VPP_UNKNOWN)
-    return TRUE;
-
-  display = GST_VAAPI_PLUGIN_BASE_DISPLAY (vaapidecbin->decoder);
-  if (display) {
-    GST_INFO_OBJECT (vaapidecbin, "Got display from vaapidecode");
-    gst_vaapi_display_ref (display);
-  } else {
-    GST_INFO_OBJECT (vaapidecbin, "Creating a dummy display to test for vpp");
-    display = gst_vaapi_create_test_display ();
-  }
-  if (!display)
-    return FALSE;
-
-  vaapidecbin->has_vpp = gst_vaapi_display_has_video_processing (display) ?
-      HAS_VPP_YES : HAS_VPP_NO;
-
-  gst_vaapi_display_unref (display);
-
-  return TRUE;
-}
-
-static gboolean
-gst_vaapi_decode_bin_reconfigure (GstVaapiDecodeBin * vaapidecbin)
-{
-  if (!ensure_vpp (vaapidecbin))
-    return FALSE;
-
-  return activate_vpp (vaapidecbin);
+  GST_ELEMENT_WARNING (vaapidecbin, CORE, MISSING_PLUGIN,
+      ("Missing element '%s' - check your GStreamer installation.",
+          missing_factory), ("video decoding might fail"));
 }
 
 static void
@@ -276,19 +168,9 @@ gst_vaapi_decode_bin_set_property (GObject * object,
             vaapidecbin->deinterlace_method, NULL);
       break;
     case PROP_DISABLE_VPP:
-    {
-      gboolean disable_vpp;
-
-      disable_vpp = g_value_get_boolean (value);
-      if (!disable_vpp && !vaapidecbin->has_vpp)
-        GST_WARNING_OBJECT (vaapidecbin,
-            "Cannot enable VPP since the VA driver does not support it");
-      else
-        vaapidecbin->disable_vpp = disable_vpp;
-
       /* @TODO: Add run-time disabling support */
+      vaapidecbin->disable_vpp = g_value_get_boolean (value);
       break;
-    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -323,101 +205,18 @@ gst_vaapi_decode_bin_get_property (GObject * object,
   }
 }
 
-static void
-gst_vaapi_decode_bin_handle_message (GstBin * bin, GstMessage * message)
-{
-  GstVaapiDecodeBin *vaapidecbin = GST_VAAPI_DECODE_BIN (bin);
-  GstMessageType type;
-  GstContext *context = NULL;
-  const gchar *context_type;
-  GstVaapiDisplay *display = NULL;
-
-  type = GST_MESSAGE_TYPE (message);
-  if (type != GST_MESSAGE_HAVE_CONTEXT)
-    goto bail;
-  gst_message_parse_have_context (message, &context);
-  if (!context)
-    goto bail;
-  context_type = gst_context_get_context_type (context);
-  if (g_strcmp0 (context_type, GST_VAAPI_DISPLAY_CONTEXT_TYPE_NAME) != 0)
-    goto bail;
-  if (!gst_vaapi_video_context_get_display (context, &display))
-    goto bail;
-
-  vaapidecbin->has_vpp = gst_vaapi_display_has_video_processing (display) ?
-      HAS_VPP_YES : HAS_VPP_NO;
-
-  /* the underlying VA driver implementation doesn't support video
-   * post-processing, hence we have to disable it */
-  if (vaapidecbin->has_vpp != HAS_VPP_YES) {
-    GST_WARNING_OBJECT (vaapidecbin, "VA driver doesn't support VPP");
-    if (!vaapidecbin->disable_vpp) {
-      vaapidecbin->disable_vpp = TRUE;
-      g_object_notify_by_pspec (G_OBJECT (vaapidecbin),
-          properties[PROP_DISABLE_VPP]);
-    }
-  }
-
-bail:
-  if (display)
-    gst_vaapi_display_unref (display);
-
-  if (context)
-    gst_context_unref (context);
-
-  GST_BIN_CLASS (gst_vaapi_decode_bin_parent_class)->handle_message (bin,
-      message);
-}
-
-static GstStateChangeReturn
-gst_vaapi_decode_bin_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstVaapiDecodeBin *vaapidecbin = GST_VAAPI_DECODE_BIN (element);
-  GstStateChangeReturn ret;
-
-  switch (transition) {
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (gst_vaapi_decode_bin_parent_class)->change_state
-      (element, transition);
-  if (ret == GST_STATE_CHANGE_FAILURE)
-    return ret;
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      if (!gst_vaapi_decode_bin_reconfigure (vaapidecbin))
-        return GST_STATE_CHANGE_FAILURE;
-      break;
-    default:
-      break;
-  }
-
-  return ret;
-}
-
 static void
 gst_vaapi_decode_bin_class_init (GstVaapiDecodeBinClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class;
-  GstBinClass *bin_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
   element_class = GST_ELEMENT_CLASS (klass);
-  bin_class = GST_BIN_CLASS (klass);
 
   gobject_class->set_property = gst_vaapi_decode_bin_set_property;
   gobject_class->get_property = gst_vaapi_decode_bin_get_property;
 
-  element_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_vaapi_decode_bin_change_state);
-
-  bin_class->handle_message =
-      GST_DEBUG_FUNCPTR (gst_vaapi_decode_bin_handle_message);
-
   gst_element_class_set_static_metadata (element_class,
       "VA-API Decode Bin",
       "Codec/Decoder/Video",
@@ -448,11 +247,11 @@ gst_vaapi_decode_bin_class_init (GstVaapiDecodeBinClass * klass)
 
   g_object_class_install_properties (gobject_class, PROP_LAST, properties);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vaapi_decode_bin_sink_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vaapi_decode_bin_sink_factory);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vaapi_decode_bin_src_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vaapi_decode_bin_src_factory);
 
   GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi_decode_bin,
       GST_PLUGIN_NAME, 0, GST_PLUGIN_DESC);
@@ -463,22 +262,19 @@ gst_vaapi_decode_bin_configure (GstVaapiDecodeBin * vaapidecbin)
 {
   const gchar *missing_factory = NULL;
   GstPad *pad, *ghostpad;
-  GstPadTemplate *tmpl;
 
   /* create the decoder */
-  /* @FIXME: "vaapidecode" is going to be removed soon: (bug
-   * #734093). Instead there are going to be a set of elements
-   * "vaapi{codec}dec". We will need a mechanism to automatically
-   * select de correct decoder based on caps.
-   */
   vaapidecbin->decoder =
-      gst_element_factory_make ("vaapidecode", "vaapidecode");
-  if (!vaapidecbin->decoder) {
-    missing_factory = "vaapidecode";
-    goto error_element_missing;
+      g_object_new (g_type_from_name ("GstVaapiDecode"), NULL);
+
+  if (vaapidecbin->disable_vpp) {
+    gst_bin_add (GST_BIN (vaapidecbin), vaapidecbin->decoder);
+    pad = gst_element_get_static_pad (vaapidecbin->decoder, "src");
+    goto bail;
   }
+
   /* create the queue */
-  vaapidecbin->queue = gst_element_factory_make ("queue", "queue");
+  vaapidecbin->queue = gst_element_factory_make ("queue", "vaapi-queue");
   if (!vaapidecbin->queue) {
     missing_factory = "queue";
     goto error_element_missing;
@@ -489,12 +285,30 @@ gst_vaapi_decode_bin_configure (GstVaapiDecodeBin * vaapidecbin)
       "max-size-buffers", vaapidecbin->max_size_buffers,
       "max-size-time", vaapidecbin->max_size_time, NULL);
 
-  gst_bin_add_many (GST_BIN (vaapidecbin), vaapidecbin->decoder,
-      vaapidecbin->queue, NULL);
+  /* create the postproc */
+  vaapidecbin->postproc = gst_element_factory_make ("vaapipostproc", NULL);
+  if (!vaapidecbin->postproc) {
+    missing_factory = "vaapipostproc";
+    goto error_element_missing;
+  }
 
-  if (!gst_element_link_many (vaapidecbin->decoder, vaapidecbin->queue, NULL))
+  gst_bin_add_many (GST_BIN (vaapidecbin), vaapidecbin->decoder,
+      vaapidecbin->queue, vaapidecbin->postproc, NULL);
+
+  if (!gst_element_link_many (vaapidecbin->decoder, vaapidecbin->queue,
+          vaapidecbin->postproc, NULL))
     goto error_link_pad;
 
+  /* create ghost pad src */
+  pad = gst_element_get_static_pad (GST_ELEMENT (vaapidecbin->postproc), "src");
+
+bail:
+  ghostpad = gst_ghost_pad_new_from_template ("src", pad,
+      GST_PAD_PAD_TEMPLATE (pad));
+  gst_object_unref (pad);
+  if (!gst_element_add_pad (GST_ELEMENT (vaapidecbin), ghostpad))
+    goto error_adding_pad;
+
   /* create ghost pad sink */
   pad = gst_element_get_static_pad (GST_ELEMENT (vaapidecbin->decoder), "sink");
   ghostpad = gst_ghost_pad_new_from_template ("sink", pad,
@@ -503,15 +317,6 @@ gst_vaapi_decode_bin_configure (GstVaapiDecodeBin * vaapidecbin)
   if (!gst_element_add_pad (GST_ELEMENT (vaapidecbin), ghostpad))
     goto error_adding_pad;
 
-  /* create ghost pad src */
-  pad = gst_element_get_static_pad (GST_ELEMENT (vaapidecbin->queue), "src");
-  tmpl = gst_static_pad_template_get (&gst_vaapi_decode_bin_src_factory);
-  ghostpad = gst_ghost_pad_new_from_template ("src", pad, tmpl);
-  gst_object_unref (pad);
-  gst_object_unref (tmpl);
-  if (!gst_element_add_pad (GST_ELEMENT (vaapidecbin), ghostpad))
-    goto error_adding_pad;
-
   return TRUE;
 
 error_element_missing:
@@ -536,7 +341,6 @@ error_adding_pad:
 static void
 gst_vaapi_decode_bin_init (GstVaapiDecodeBin * vaapidecbin)
 {
-  vaapidecbin->has_vpp = HAS_VPP_UNKNOWN;
   vaapidecbin->deinterlace_method = DEFAULT_DEINTERLACE_METHOD;
 
   gst_vaapi_decode_bin_configure (vaapidecbin);
