@@ -42,6 +42,9 @@
 #include "gstvaapipluginutil.h"
 #include "gstvaapipluginbase.h"
 
+/* Environment variable for disable driver white-list */
+#define GST_VAAPI_ALL_DRIVERS_ENV "GST_VAAPI_ALL_DRIVERS"
+
 typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFunc) (const gchar *);
 typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFromHandleFunc) (gpointer);
 
@@ -133,12 +136,19 @@ gst_vaapi_create_display_from_gl_context (GstObject * gl_context_object)
   GstGLDisplay *const gl_display = gst_gl_context_get_display (gl_context);
   gpointer native_display =
       GSIZE_TO_POINTER (gst_gl_display_get_handle (gl_display));
+  GstGLPlatform platform = gst_gl_context_get_gl_platform (gl_context);
   GstVaapiDisplay *display, *out_display;
   GstVaapiDisplayType display_type;
 
   switch (gst_gl_display_get_handle_type (gl_display)) {
 #if USE_X11
     case GST_GL_DISPLAY_TYPE_X11:
+#if USE_GLX
+      if (platform == GST_GL_PLATFORM_GLX) {
+        display_type = GST_VAAPI_DISPLAY_TYPE_GLX;
+        break;
+      }
+#endif
       display_type = GST_VAAPI_DISPLAY_TYPE_X11;
       break;
 #endif
@@ -189,7 +199,7 @@ gst_vaapi_create_display_from_gl_context (GstObject * gl_context_object)
   if (!display)
     return NULL;
 
-  switch (gst_gl_context_get_gl_platform (gl_context)) {
+  switch (platform) {
 #if USE_EGL
     case GST_GL_PLATFORM_EGL:{
       guint gles_version;
@@ -432,55 +442,72 @@ gst_vaapi_video_format_new_template_caps_with_features (GstVideoFormat format,
   return caps;
 }
 
-static GstCaps *
-new_gl_texture_upload_meta_caps (void)
+static GstVideoFormat
+gst_vaapi_find_preferred_format (const GValue * format_list,
+    GstVideoFormat native_format)
 {
-  return
-      gst_caps_from_string (GST_VIDEO_CAPS_MAKE_WITH_FEATURES
-      (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
-          "{ RGBA, BGRA }"));
+  const GValue *frmt;
+  GstVideoFormat out_format;
+  guint i;
+
+  /* if one format, that is the one */
+  if (G_VALUE_HOLDS_STRING (format_list))
+    return gst_video_format_from_string (g_value_get_string (format_list));
+
+  if (!GST_VALUE_HOLDS_LIST (format_list)) {
+    GST_ERROR ("negotiated caps do not have a valid format");
+    return GST_VIDEO_FORMAT_UNKNOWN;
+  }
+
+  if (native_format == GST_VIDEO_FORMAT_UNKNOWN
+      || native_format == GST_VIDEO_FORMAT_ENCODED) {
+    native_format = GST_VIDEO_FORMAT_NV12;      /* default VA format */
+  }
+
+  /* search our native format in the list */
+  for (i = 0; i < gst_value_list_get_size (format_list); i++) {
+    frmt = gst_value_list_get_value (format_list, i);
+    out_format = gst_video_format_from_string (g_value_get_string (frmt));
+
+    /* GStreamer do not handle encoded formats nicely. Try the next
+     * one. */
+    if (out_format == GST_VIDEO_FORMAT_ENCODED)
+      continue;
+
+    if (native_format == out_format)
+      return out_format;
+  }
+
+  /* just pick the first valid format in the list */
+  i = 0;
+  do {
+    frmt = gst_value_list_get_value (format_list, i++);
+    out_format = gst_video_format_from_string (g_value_get_string (frmt));
+  } while (out_format == GST_VIDEO_FORMAT_ENCODED);
+
+  return out_format;
 }
 
 GstVaapiCapsFeature
-gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format,
+gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstCaps * allowed_caps,
     GstVideoFormat * out_format_ptr)
 {
-  GstVaapiCapsFeature feature = GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY;
-  guint i, num_structures;
-  GstCaps *caps = NULL;
-  GstCaps *gl_texture_upload_caps = NULL;
-  GstCaps *sysmem_caps = NULL;
-  GstCaps *vaapi_caps = NULL;
-  GstCaps *out_caps, *templ;
-  GstVideoFormat out_format;
+  GstVaapiCapsFeature feature = GST_VAAPI_CAPS_FEATURE_NOT_NEGOTIATED;
+  guint i, j, num_structures;
+  GstCaps *out_caps, *caps = NULL;
+  static const guint feature_list[] = { GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE,
+    GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META,
+    GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY,
+  };
 
-  templ = gst_pad_get_pad_template_caps (pad);
-  out_caps = gst_pad_peer_query_caps (pad, templ);
-  gst_caps_unref (templ);
-  if (!out_caps) {
-    feature = GST_VAAPI_CAPS_FEATURE_NOT_NEGOTIATED;
-    goto cleanup;
-  }
-
-  out_format = format == GST_VIDEO_FORMAT_ENCODED ?
-      GST_VIDEO_FORMAT_I420 : format;
-
-  gl_texture_upload_caps = new_gl_texture_upload_meta_caps ();
-  if (!gl_texture_upload_caps)
+  out_caps = gst_pad_peer_query_caps (pad, allowed_caps);
+  if (!out_caps)
     goto cleanup;
 
-  vaapi_caps =
-      gst_vaapi_video_format_new_template_caps_with_features (out_format,
-      GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
-  if (!vaapi_caps)
-    goto cleanup;
-
-  sysmem_caps =
-      gst_vaapi_video_format_new_template_caps_with_features (out_format,
-      GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
-  if (!sysmem_caps)
+  if (gst_caps_is_any (out_caps) || gst_caps_is_empty (out_caps))
     goto cleanup;
 
+  feature = GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY;
   num_structures = gst_caps_get_size (out_caps);
   for (i = 0; i < num_structures; i++) {
     GstCapsFeatures *const features = gst_caps_get_features (out_caps, i);
@@ -490,21 +517,19 @@ gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format,
     if (gst_caps_features_is_any (features))
       continue;
 
+    gst_caps_replace (&caps, NULL);
     caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
     if (!caps)
       continue;
     gst_caps_set_features (caps, 0, gst_caps_features_copy (features));
 
-    if (gst_caps_can_intersect (caps, vaapi_caps) &&
-        feature < GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE)
-      feature = GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE;
-    else if (gst_caps_can_intersect (caps, gl_texture_upload_caps) &&
-        feature < GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META)
-      feature = GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META;
-    else if (gst_caps_can_intersect (caps, sysmem_caps) &&
-        feature < GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY)
-      feature = GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY;
-    gst_caps_replace (&caps, NULL);
+    for (j = 0; j < G_N_ELEMENTS (feature_list); j++) {
+      if (gst_vaapi_caps_feature_contains (caps, feature_list[j])
+          && feature < feature_list[j]) {
+        feature = feature_list[j];
+        break;
+      }
+    }
 
     /* Stop at the first match, the caps should already be sorted out
        by preference order from downstream elements */
@@ -512,35 +537,35 @@ gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format,
       break;
   }
 
+  if (!caps)
+    goto cleanup;
+
   if (out_format_ptr) {
-    if (feature == GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META) {
-      GstStructure *structure;
-      gchar *format_str;
-      out_format = GST_VIDEO_FORMAT_UNKNOWN;
-      do {
-        caps = gst_caps_intersect_full (out_caps, gl_texture_upload_caps,
-            GST_CAPS_INTERSECT_FIRST);
-        if (!caps)
-          break;
-        structure = gst_caps_get_structure (caps, 0);
-        if (!structure)
-          break;
-        if (!gst_structure_get (structure, "format", G_TYPE_STRING,
-                &format_str, NULL))
-          break;
-        out_format = gst_video_format_from_string (format_str);
-        g_free (format_str);
-      } while (0);
-      if (!out_format)
-        goto cleanup;
-    }
+    GstVideoFormat out_format;
+    GstStructure *structure;
+    const GValue *format_list;
+
+    /* if the best feature is SystemMemory, we should use the first
+     * caps in the peer caps set, which is the preferred by
+     * downstream. */
+    if (feature == GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY)
+      gst_caps_replace (&caps, out_caps);
+
+    /* use the first caps, which is the preferred by downstream. */
+    structure = gst_caps_get_structure (caps, 0);
+    if (!structure)
+      goto cleanup;
+    format_list = gst_structure_get_value (structure, "format");
+    if (!format_list)
+      goto cleanup;
+    out_format = gst_vaapi_find_preferred_format (format_list, *out_format_ptr);
+    if (out_format == GST_VIDEO_FORMAT_UNKNOWN)
+      goto cleanup;
+
     *out_format_ptr = out_format;
   }
 
 cleanup:
-  gst_caps_replace (&gl_texture_upload_caps, NULL);
-  gst_caps_replace (&sysmem_caps, NULL);
-  gst_caps_replace (&vaapi_caps, NULL);
   gst_caps_replace (&caps, NULL);
   gst_caps_replace (&out_caps, NULL);
   return feature;
@@ -644,18 +669,55 @@ gst_video_info_change_format (GstVideoInfo * vip, GstVideoFormat format,
 
   gst_video_info_set_format (vip, format, width, height);
 
-  vip->interlace_mode = vi.interlace_mode;
-  vip->flags = vi.flags;
-  vip->views = vi.views;
-  vip->par_n = vi.par_n;
-  vip->par_d = vi.par_d;
-  vip->fps_n = vi.fps_n;
-  vip->fps_d = vi.fps_d;
-
+  GST_VIDEO_INFO_INTERLACE_MODE (vip) = GST_VIDEO_INFO_INTERLACE_MODE (&vi);
+  GST_VIDEO_FORMAT_INFO_FLAGS (vip) = GST_VIDEO_FORMAT_INFO_FLAGS (&vi);
+  GST_VIDEO_INFO_VIEWS (vip) = GST_VIDEO_INFO_VIEWS (&vi);
+  GST_VIDEO_INFO_PAR_N (vip) = GST_VIDEO_INFO_PAR_N (&vi);
+  GST_VIDEO_INFO_PAR_D (vip) = GST_VIDEO_INFO_PAR_D (&vi);
+  GST_VIDEO_INFO_FPS_N (vip) = GST_VIDEO_INFO_FPS_N (&vi);
+  GST_VIDEO_INFO_FPS_D (vip) = GST_VIDEO_INFO_FPS_D (&vi);
   GST_VIDEO_INFO_MULTIVIEW_MODE (vip) = GST_VIDEO_INFO_MULTIVIEW_MODE (&vi);
   GST_VIDEO_INFO_MULTIVIEW_FLAGS (vip) = GST_VIDEO_INFO_MULTIVIEW_FLAGS (&vi);
 }
 
+/**
+ * gst_video_info_changed:
+ * @old: old #GstVideoInfo
+ * @new: new #GstVideoInfo
+ *
+ * Compares @old and @new
+ *
+ * Returns: %TRUE if @old has different format/width/height than
+ * @new. Otherwise, %FALSE.
+ **/
+gboolean
+gst_video_info_changed (const GstVideoInfo * old, const GstVideoInfo * new)
+{
+  if (GST_VIDEO_INFO_FORMAT (old) != GST_VIDEO_INFO_FORMAT (new))
+    return TRUE;
+  if (GST_VIDEO_INFO_WIDTH (old) != GST_VIDEO_INFO_WIDTH (new))
+    return TRUE;
+  if (GST_VIDEO_INFO_HEIGHT (old) != GST_VIDEO_INFO_HEIGHT (new))
+    return TRUE;
+  return FALSE;
+}
+
+/**
+ * gst_video_info_force_nv12_if_encoded:
+ * @vinfo: a #GstVideoInfo
+ *
+ * If the format of @vinfo is %GST_VIDEO_FORMAT_ENCODED it is changed
+ * to %GST_VIDEO_FORMAT_NV12.
+ **/
+void
+gst_video_info_force_nv12_if_encoded (GstVideoInfo * vinfo)
+{
+  if (GST_VIDEO_INFO_FORMAT (vinfo) != GST_VIDEO_FORMAT_ENCODED)
+    return;
+  gst_video_info_set_format (vinfo, GST_VIDEO_FORMAT_NV12,
+      GST_VIDEO_INFO_WIDTH (vinfo), GST_VIDEO_INFO_HEIGHT (vinfo));
+}
+
 /**
  * gst_vaapi_create_test_display:
  *
@@ -670,3 +732,73 @@ gst_vaapi_create_test_display (void)
 {
   return gst_vaapi_create_display (GST_VAAPI_DISPLAY_TYPE_ANY, NULL);
 }
+
+/**
+ * gst_vaapi_driver_is_whitelisted:
+ * @display: a #GstVaapiDisplay
+ *
+ * Looks the VA-API driver vendors in an internal white-list.
+ *
+ * Returns: %TRUE if driver is in the white-list, otherwise %FALSE
+ **/
+gboolean
+gst_vaapi_driver_is_whitelisted (GstVaapiDisplay * display)
+{
+  const gchar *vendor;
+  guint i;
+  static const gchar *whitelist[] = {
+    "Intel i965 driver",
+    "mesa gallium vaapi",
+    NULL
+  };
+
+  g_return_val_if_fail (display, FALSE);
+
+  if (g_getenv (GST_VAAPI_ALL_DRIVERS_ENV))
+    return TRUE;
+
+  vendor = gst_vaapi_display_get_vendor_string (display);
+  if (!vendor)
+    goto no_vendor;
+
+  for (i = 0; whitelist[i]; i++) {
+    if (g_ascii_strncasecmp (vendor, whitelist[i], strlen (whitelist[i])) == 0)
+      return TRUE;
+  }
+
+  GST_ERROR ("Unsupported VA driver: %s. Export environment variable "
+      GST_VAAPI_ALL_DRIVERS_ENV " to bypass", vendor);
+  return FALSE;
+
+  /* ERRORS */
+no_vendor:
+  {
+    GST_WARNING ("no VA-API driver vendor description");
+    return FALSE;
+  }
+}
+
+/**
+ * gst_vaapi_codecs_has_codec:
+ * @codecs: a #GArray of #GstVaapiCodec
+ * @codec: a #GstVaapiCodec to find in @codec
+ *
+ * Search in the available @codecs for the specific @codec.
+ *
+ * Returns: %TRUE if @codec is in @codecs
+ **/
+gboolean
+gst_vaapi_codecs_has_codec (GArray * codecs, GstVaapiCodec codec)
+{
+  guint i;
+  GstVaapiCodec c;
+
+  g_return_val_if_fail (codec, FALSE);
+
+  for (i = 0; i < codecs->len; i++) {
+    c = g_array_index (codecs, GstVaapiCodec, i);
+    if (c == codec)
+      return TRUE;
+  }
+  return FALSE;
+}
