@@ -160,11 +160,11 @@ _fixate_frame_size (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
   to_par = gst_structure_get_value (outs, "pixel-aspect-ratio");
   if (!to_par) {
     g_value_init (&tpar, GST_TYPE_FRACTION);
-    gst_value_set_fraction (&tpar, 1, 1);
+    gst_value_set_fraction (&tpar, GST_VIDEO_INFO_PAR_N (vinfo),
+        GST_VIDEO_INFO_PAR_D (vinfo));
     to_par = &tpar;
 
-    gst_structure_set (outs, "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
-        NULL);
+    gst_structure_set_value (outs, "pixel-aspect-ratio", &tpar);
   }
 
   /* we have both PAR but they might not be fixated */
@@ -585,6 +585,28 @@ _set_colorimetry (GstVaapiPostproc * postproc, GstVideoFormat format,
   return TRUE;
 }
 
+static gboolean
+_set_interlace_mode (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
+    GstStructure * outs)
+{
+  const gchar *interlace_mode = NULL;
+
+  if (is_deinterlace_enabled (postproc, vinfo)) {
+    interlace_mode = "progressive";
+  } else {
+    interlace_mode =
+        gst_video_interlace_mode_to_string (GST_VIDEO_INFO_INTERLACE_MODE
+        (vinfo));
+  }
+
+  if (!interlace_mode)
+    return FALSE;
+
+  gst_structure_set (outs, "interlace-mode", G_TYPE_STRING, interlace_mode,
+      NULL);
+  return TRUE;
+}
+
 static gboolean
 _set_preferred_format (GstStructure * outs, GstVideoFormat format)
 {
@@ -649,6 +671,9 @@ _get_preferred_caps (GstVaapiPostproc * postproc, GstVideoInfo * vinfo,
   if (f == GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY)
     _set_colorimetry (postproc, format, structure);
 
+  if (!_set_interlace_mode (postproc, vinfo, structure))
+    goto interlace_mode_failed;
+
   outcaps = gst_caps_new_empty ();
   gst_caps_append_structure_full (outcaps, structure,
       gst_caps_features_copy (features));
@@ -673,6 +698,11 @@ invalid_caps:
     GST_WARNING_OBJECT (postproc, "No valid src caps found");
     return NULL;
   }
+interlace_mode_failed:
+  {
+    GST_WARNING_OBJECT (postproc, "Invalid sink caps interlace mode");
+    return NULL;
+  }
 }
 
 /**
