@@ -24,6 +24,7 @@
 #include "gstvaapivideobufferpool.h"
 #include "gstvaapivideobuffer.h"
 #include "gstvaapivideomemory.h"
+#include "gstvaapipluginutil.h"
 #if (USE_GLX || USE_EGL)
 #include "gstvaapivideometa_texture.h"
 #endif
@@ -43,14 +44,11 @@ enum
 
 struct _GstVaapiVideoBufferPoolPrivate
 {
-  GstVideoInfo video_info[2];
-  guint video_info_index;
+  GstVideoInfo video_info;
   GstAllocator *allocator;
   GstVideoInfo alloc_info;
   GstVaapiDisplay *display;
-  guint has_video_meta:1;
-  guint has_video_alignment:1;
-  guint has_texture_upload_meta:1;
+  guint options;
   guint use_dmabuf_memory:1;
 };
 
@@ -136,69 +134,65 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
 {
   GstVaapiVideoBufferPoolPrivate *const priv =
       GST_VAAPI_VIDEO_BUFFER_POOL (pool)->priv;
-  GstCaps *caps = NULL;
-  GstVideoInfo *const cur_vip = &priv->video_info[priv->video_info_index];
-  GstVideoInfo *const new_vip = &priv->video_info[!priv->video_info_index];
+  GstCaps *caps;
+  GstVideoInfo new_vip;
+  const GstVideoInfo *alloc_vip;
   GstVideoAlignment align;
   GstAllocator *allocator;
-  gboolean changed_caps, use_dmabuf_memory, ret, updated = FALSE;
+  gboolean ret, updated = FALSE;
 
+  GST_DEBUG_OBJECT (pool, "config %" GST_PTR_FORMAT, config);
+
+  caps = NULL;
   if (!gst_buffer_pool_config_get_params (config, &caps, NULL, NULL, NULL))
     goto error_invalid_config;
-  if (!caps || !gst_video_info_from_caps (new_vip, caps))
+  if (!caps)
     goto error_no_caps;
+  if (!gst_video_info_from_caps (&new_vip, caps))
+    goto error_invalid_caps;
 
-  use_dmabuf_memory = gst_buffer_pool_config_has_option (config,
-      GST_BUFFER_POOL_OPTION_DMABUF_MEMORY);
-  if (priv->use_dmabuf_memory != use_dmabuf_memory) {
-    priv->use_dmabuf_memory = use_dmabuf_memory;
-    g_clear_object (&priv->allocator);
-  }
+  allocator = NULL;
+  if (!gst_buffer_pool_config_get_allocator (config, &allocator, NULL))
+    goto error_invalid_allocator;
 
-  changed_caps = !priv->allocator ||
-      GST_VIDEO_INFO_FORMAT (cur_vip) != GST_VIDEO_INFO_FORMAT (new_vip) ||
-      GST_VIDEO_INFO_WIDTH (cur_vip) != GST_VIDEO_INFO_WIDTH (new_vip) ||
-      GST_VIDEO_INFO_HEIGHT (cur_vip) != GST_VIDEO_INFO_HEIGHT (new_vip);
-
-  if (changed_caps) {
-    const GstVideoInfo *alloc_vip;
-    guint flags = 0;
-
-    if (use_dmabuf_memory) {
-      /* XXX: also needs fixed strides/offsets */
-      flags |= GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE;
-      allocator =
-          gst_vaapi_dmabuf_allocator_new (priv->display, new_vip, flags);
-    } else {
-      allocator = gst_vaapi_video_allocator_new (priv->display, new_vip, 0);
-    }
-    if (!allocator)
-      goto error_create_allocator;
-    gst_object_replace ((GstObject **) & priv->allocator,
-        GST_OBJECT_CAST (allocator));
-    gst_object_unref (allocator);
-    priv->video_info_index ^= 1;
-
-    alloc_vip = gst_allocator_get_vaapi_video_info (allocator, NULL);
-    if (!alloc_vip)
-      goto error_create_allocator_info;
-    priv->alloc_info = *alloc_vip;
-  }
+  if (gst_video_info_changed (&priv->video_info, &new_vip))
+    gst_object_replace ((GstObject **) & priv->allocator, NULL);
+  priv->video_info = new_vip;
 
   if (!gst_buffer_pool_config_has_option (config,
           GST_BUFFER_POOL_OPTION_VAAPI_VIDEO_META))
     goto error_no_vaapi_video_meta_option;
 
-  priv->has_video_meta = gst_buffer_pool_config_has_option (config,
-      GST_BUFFER_POOL_OPTION_VIDEO_META);
-  if (!priv->has_video_meta) {
+  /* not our allocator, not our buffers */
+  if (allocator) {
+    priv->use_dmabuf_memory = gst_vaapi_is_dmabuf_allocator (allocator);
+    if (priv->use_dmabuf_memory ||
+        g_strcmp0 (allocator->mem_type, GST_VAAPI_VIDEO_MEMORY_NAME) == 0) {
+      if (priv->allocator)
+        gst_object_unref (priv->allocator);
+      if ((priv->allocator = allocator))
+        gst_object_ref (allocator);
+      alloc_vip = gst_allocator_get_vaapi_video_info (priv->allocator, NULL);
+      if (!alloc_vip)
+        goto error_create_allocator_info;
+      priv->alloc_info = *alloc_vip;
+    }
+  }
+  if (!priv->allocator)
+    goto error_no_allocator;
+
+  priv->options = 0;
+  if (gst_buffer_pool_config_has_option (config,
+          GST_BUFFER_POOL_OPTION_VIDEO_META))
+    priv->options |= GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META;
+  else {
     gint i;
-    for (i = 0; i < GST_VIDEO_INFO_N_PLANES (new_vip); i++) {
-      if (GST_VIDEO_INFO_PLANE_OFFSET (new_vip, i) !=
+    for (i = 0; i < GST_VIDEO_INFO_N_PLANES (&priv->video_info); i++) {
+      if (GST_VIDEO_INFO_PLANE_OFFSET (&priv->video_info, i) !=
           GST_VIDEO_INFO_PLANE_OFFSET (&priv->alloc_info, i) ||
-          GST_VIDEO_INFO_PLANE_STRIDE (new_vip, i) !=
+          GST_VIDEO_INFO_PLANE_STRIDE (&priv->video_info, i) !=
           GST_VIDEO_INFO_PLANE_STRIDE (&priv->alloc_info, i)) {
-        priv->has_video_meta = TRUE;
+        priv->options |= GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META;
         gst_buffer_pool_config_add_option (config,
             GST_BUFFER_POOL_OPTION_VIDEO_META);
         updated = TRUE;
@@ -207,15 +201,15 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
     }
   }
 
-  priv->has_video_alignment = gst_buffer_pool_config_has_option (config,
-      GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
-  if (priv->has_video_alignment) {
+  if (gst_buffer_pool_config_has_option (config,
+          GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT)) {
     fill_video_alignment (GST_VAAPI_VIDEO_BUFFER_POOL (pool), &align);
     gst_buffer_pool_config_set_video_alignment (config, &align);
   }
 
-  priv->has_texture_upload_meta = gst_buffer_pool_config_has_option (config,
-      GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
+  if (!priv->use_dmabuf_memory && gst_buffer_pool_config_has_option (config,
+          GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META))
+    priv->options |= GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_GL_TEXTURE_UPLOAD;
 
   ret =
       GST_BUFFER_POOL_CLASS
@@ -225,27 +219,38 @@ gst_vaapi_video_buffer_pool_set_config (GstBufferPool * pool,
   /* ERRORS */
 error_invalid_config:
   {
-    GST_ERROR ("invalid config");
+    GST_ERROR_OBJECT (pool, "invalid config");
     return FALSE;
   }
 error_no_caps:
   {
-    GST_ERROR ("no valid caps in config");
+    GST_ERROR_OBJECT (pool, "no caps in config");
     return FALSE;
   }
-error_create_allocator:
+error_invalid_caps:
   {
-    GST_ERROR ("failed to create GstVaapiVideoAllocator object");
+    GST_ERROR_OBJECT (pool, "invalid caps %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
-error_create_allocator_info:
+error_invalid_allocator:
   {
-    GST_ERROR ("failed to create GstVaapiVideoAllocator `video-info'");
+    GST_ERROR_OBJECT (pool, "no allocator in config");
     return FALSE;
   }
 error_no_vaapi_video_meta_option:
   {
-    GST_ERROR ("no GstVaapiVideoMeta option");
+    GST_ERROR_OBJECT (pool, "no GstVaapiVideoMeta option in config");
+    return FALSE;
+  }
+error_create_allocator_info:
+  {
+    GST_ERROR_OBJECT (pool,
+        "failed to create GstVaapiVideoAllocator `video-info'");
+    return FALSE;
+  }
+error_no_allocator:
+  {
+    GST_ERROR_OBJECT (pool, "no allocator defined");
     return FALSE;
   }
 }
@@ -288,7 +293,7 @@ gst_vaapi_video_buffer_pool_alloc_buffer (GstBufferPool * pool,
   gst_vaapi_video_meta_replace (&meta, NULL);
   gst_buffer_append_memory (buffer, mem);
 
-  if (priv->has_video_meta) {
+  if (priv->options & GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_VIDEO_META) {
     GstVideoInfo *const vip = &priv->alloc_info;
     GstVideoMeta *vmeta;
 
@@ -304,7 +309,7 @@ gst_vaapi_video_buffer_pool_alloc_buffer (GstBufferPool * pool,
     }
   }
 #if (USE_GLX || USE_EGL)
-  if (priv->has_texture_upload_meta)
+  if (priv->options & GST_VAAPI_VIDEO_BUFFER_POOL_OPTION_GL_TEXTURE_UPLOAD)
     gst_buffer_add_texture_upload_meta (buffer);
 #endif
 
@@ -314,23 +319,23 @@ gst_vaapi_video_buffer_pool_alloc_buffer (GstBufferPool * pool,
   /* ERRORS */
 error_no_allocator:
   {
-    GST_ERROR ("no GstAllocator in buffer pool");
+    GST_ERROR_OBJECT (pool, "no GstAllocator in buffer pool");
     return GST_FLOW_ERROR;
   }
 error_create_meta:
   {
-    GST_ERROR ("failed to allocate vaapi video meta");
+    GST_ERROR_OBJECT (pool, "failed to allocate vaapi video meta");
     return GST_FLOW_ERROR;
   }
 error_create_buffer:
   {
-    GST_ERROR ("failed to create video buffer");
+    GST_ERROR_OBJECT (pool, "failed to create video buffer");
     gst_vaapi_video_meta_unref (meta);
     return GST_FLOW_ERROR;
   }
 error_create_memory:
   {
-    GST_ERROR ("failed to create video memory");
+    GST_ERROR_OBJECT (pool, "failed to create video memory");
     gst_buffer_unref (buffer);
     gst_vaapi_video_meta_unref (meta);
     return GST_FLOW_ERROR;
@@ -392,9 +397,7 @@ gst_vaapi_video_buffer_pool_init (GstVaapiVideoBufferPool * pool)
 
   pool->priv = priv;
 
-  gst_video_info_init (&priv->video_info[0]);
-  gst_video_info_init (&priv->video_info[1]);
-  gst_video_info_init (&priv->alloc_info);
+  gst_video_info_init (&priv->video_info);
 }
 
 GstBufferPool *
